"""Module for commonly used paths."""

from pathlib import Path

SOURCE_ROOT_PATH = Path(__file__).parent
"""* A path to the source code directory."""

PROJECT_ROOT_PATH = SOURCE_ROOT_PATH.parent
"""* A path to the project root directory."""

TEST_PATH = PROJECT_ROOT_PATH / "tests"
"""* A path to the test directory."""

TEMPLATE_DIRECTORY = SOURCE_ROOT_PATH / "templates"
"""* A path to the templates directory."""

TEMPORARY_FILES_DIRECTORY = PROJECT_ROOT_PATH / "temporary_files"
"""* A path to the temporary files directory."""



"""This module provides core requests_whaor functionality."""

from concurrent.futures import as_completed, ThreadPoolExecutor
from contextlib import contextmanager, ExitStack
import random
import time
from typing import Dict, List, Optional

from loguru import logger
import requests
from requests.exceptions import (  # pylint: disable=redefined-builtin
    ConnectionError,
    ProxyError,
    Timeout,
)

from .balancer import Balancer, OnionBalancer
from .circuit import OnionCircuit, OnionCircuits
from .network import WhaorNet


def pause(sleep: int) -> None:
    """Sleep function with a little logging fun."""
    if random.random() > 0.5:
        logger.debug("Warming things up.")
    else:
        logger.debug("Just chillin for a sec.")

    time.sleep(sleep)  # let things connect


class Requestor:
    """Makes proxied web requests via a rotating proxy TOR network."""

    def __init__(
        self, onions: List[OnionCircuit], onion_balancer: Balancer, timeout: int, max_retries: int
    ) -> "Requestor":
        """Requestor __init__ method.

        Args:
            onions (List[OnionCircuit]): List of TOR containers.
            onion_balancer (Balancer): Balancer instances connected to TOR containers
                on the same network.
            timeout (int): Requests timeout.
            max_retries (int): Max number of time to retry on bad response or connection error.
        """
        self.timeout = timeout
        self.onions = onions
        self.onion_balancer = onion_balancer
        self.max_retries = max_retries

    @property
    def rotating_proxy(self) -> Dict[str, str]:
        """Rotating proxy frontend input address."""
        return self.onion_balancer.proxies

    def get(
        self, url: str, *args, **kwargs  # noqa: ANN002, ANN003
    ) -> Optional[requests.models.Response]:
        """Overload requests.get method.

        This will pass in the rotating proxy host address and timeout into the requests.get
        method. Additionally, It provides a way to automatically retry on connection failures
        and bad status_codes. Each time there is a failure it will try a new request with
        a new ip address.

        Args:
            url (str): url to send the get request.
            *args: arguments to pass to requests.get() method.
            **kwargs: keyword arguments to pass to requests.get() method.

        Returns:
            Response: If a response is found else None.
        """
        retries = self.max_retries

        kwargs.pop("proxies", None)
        kwargs.pop("timeout", None)

        while retries > 0:
            try:
                response = requests.get(
                    url, timeout=self.timeout, proxies=self.rotating_proxy, *args, **kwargs
                )
                if response.ok:
                    return response

            except (ProxyError, Timeout, ConnectionError) as error:
                logger.error(error)

            retries -= 1
            logger.debug(f"Retrying {retries} more times.")

        return None

    def restart_onions(self, with_threads: bool = True, max_threads: int = 5) -> None:
        """Restart onion containers.

        This can be useful for changing ip addresses every n requests.

        Args:
            with_threads (bool): if True uses threads to restart the containers.
            max_threads (int): How many threads to use.
        """
        if with_threads:
            with ThreadPoolExecutor(max_workers=max_threads) as executor:
                futures = [executor.submit(onion.restart) for onion in self.onions]

                for future in as_completed(futures):
                    future.result()
        else:
            for onion in self.onions:
                onion.restart()

        pause(5)


@contextmanager
def RequestsWhaor(  # pylint: disable=invalid-name, too-many-arguments
    onion_count: int = 5,
    start_with_threads: bool = True,
    max_threads: int = 5,
    timeout: int = 5,
    show_log: bool = False,
    max_retries: int = 5,
) -> Requestor:
    """Context manager which starts n amount of tor nodes behind a round robin reverse proxy.

    Args:
        onion_count (int): Number of TOR circuits to spin up.
        start_with_threads (bool): If True uses treads to spin up containers.
        max_threads (int): Max number of threads to use when spin up containers.
        timeout (int): Requests timeout.
        show_log (bool): If True shows the containers logs.
        max_retries (int): Max number of time to retry on bad response or connection error.

    Yields:
        Requestor: Makes proxied web requests via a rotating proxy TOR network.
    """
    with ExitStack() as stack:
        try:
            network = stack.enter_context(WhaorNet())
            onions = stack.enter_context(
                OnionCircuits(
                    onion_count,
                    startup_with_threads=start_with_threads,
                    max_threads=max_threads,
                    show_log=show_log,
                )
            )
            for onion in onions:
                network.connect_container(onion.container_id, onion.container_name)

            onion_balancer = stack.enter_context(
                OnionBalancer(onions=network.containers, show_log=show_log)
            )

            network.connect_container(onion_balancer.container_id, onion_balancer.container_name)

            pause(5)

            logger.info(f"Dashboard Address: {onion_balancer.dashboard_address}")

            pause(1)

            yield Requestor(
                onions=onions,
                onion_balancer=onion_balancer,
                timeout=timeout,
                max_retries=max_retries,
            )

        finally:

            stack.pop_all().close()



"""This module provides objects for managing docker network instances."""

from contextlib import contextmanager
from typing import ContextManager, List, Optional

from docker.models.containers import Container as DockerContainer
from docker.models.networks import Network as DockerNetwork
from loguru import logger

from .client import Client


class Network(Client):
    """Represents a docker network.

    Attributes:
        name (str): Network name.
        driver (str): Network driver.
        docker_network (Optional[DockerNetwork]): Holds an instance of a started docker network.
    """

    name: str
    driver: str

    docker_network: Optional[DockerNetwork]

    def connect_container(self, container_id: str, container_name: str) -> None:
        """Connect container to network and give it a reachable network alias.

        Args:
            container_id (str): The containers id.
            container_name (str): The containers name.
        """
        logger.debug(f"connecting {container_name} to the {self.network_name} network")
        self.docker_network.connect(container_id, aliases=[container_name])

    @property
    def containers(self) -> List[DockerContainer]:
        """Return list of Container objects connected to network."""
        self.docker_network.reload()
        return self.docker_network.containers

    @property
    def network_name(self) -> str:
        """Network name."""
        return self.docker_network.name

    @property
    def network_id(self) -> str:
        """Network short id."""
        return self.docker_network.short_id

    def start(self) -> None:
        """Start Docker network."""
        client = self.get_client()
        self.docker_network = client.networks.create(name=self.name, driver=self.driver)
        logger.debug(f"Network: {self.network_name} {self.network_id} Created.")

    def stop(self) -> None:
        """Stop Docker network."""
        self.docker_network.reload()

        if self.docker_network.containers:

            for container in self.docker_network.containers:
                self.docker_network.disconnect(container.name)

        self.docker_network.remove()
        logger.debug(f"Network: {self.network_name} {self.network_id} Destroyed.")


@contextmanager
# pylint: disable=invalid-name
def WhaorNet(name: str = "whaornet", driver: str = "bridge") -> ContextManager[Network]:
    """Context manager which yields a network to connect containers to.

    Args:
        name (str): Name of network.
        driver (str): Type of network drivier.

    Yields:
        Network: A Docker network.
    """
    whaornet = Network(name=name, driver=driver)

    try:
        whaornet.start()
        yield whaornet

    finally:
        whaornet.stop()



"""This module provides objects for managing TOR docker instances."""

from concurrent.futures import as_completed, ThreadPoolExecutor
from contextlib import contextmanager
from typing import ContextManager, List, Optional

from .client import ContainerBase, ContainerOptions

TOR_IMAGE = "osminogin/tor-simple:0.4.3.6"


class OnionCircuit(ContainerBase):
    """A TOR Docker Container Object.

    Attributes:
        container_options (ContainerOptions): Container Options for TOR docker instance.
    """

    container_options: ContainerOptions = ContainerOptions(image=TOR_IMAGE)


@contextmanager
def OnionCircuits(  # pylint: disable=invalid-name
    onion_count: int,
    startup_with_threads: bool = False,
    max_threads: int = 2,
    thread_pool_timeout: Optional[int] = None,
    show_log: bool = False,
) -> ContextManager[List[OnionCircuit]]:
    """Context manager which yields a list of started TOR containers.

    Takes care of starting and stopping multiple docker container instances of TOR.

    Args:
        onion_count (int): Number of TOR docker container instances to start.
        start_with_threads (bool): If True uses threads to start up the containers.
        max_threads (int): Max number of threads to use to start up the containers.
        thread_pool_timeout (Optional[int]): Timeout for ThreadPoolExecutor.
        show_log (bool): If True shows the containers logs.

    Yields:
        List[OnionCircuit]: A list of started OnionCircuit objects.
    """
    onion_circuits = [OnionCircuit() for _ in range(onion_count)]

    try:
        if startup_with_threads:
            with ThreadPoolExecutor(max_workers=max_threads) as executor:
                futures = [
                    executor.submit(circuit.start, show_log=show_log) for circuit in onion_circuits
                ]

                for future in as_completed(futures, timeout=thread_pool_timeout):
                    future.result()

        else:
            for circuit in onion_circuits:
                circuit.start(show_log=show_log)

        yield onion_circuits

    finally:
        if startup_with_threads:
            with ThreadPoolExecutor(max_workers=max_threads) as executor:
                futures = [
                    executor.submit(circuit.stop, show_log=show_log) for circuit in onion_circuits
                ]

                for future in as_completed(futures, thread_pool_timeout):
                    future.result()

        else:
            for circuit in onion_circuits:
                circuit.stop(show_log=show_log)



"""This module provides base objects to manage docker containers."""

from tempfile import _TemporaryFileWrapper as TemporaryFile
from typing import ClassVar, Dict, List, Optional

import docker
from docker.client import DockerClient
from docker.models.containers import Container
from docker.types import Mount as DockerMount
from loguru import logger
from pydantic import BaseModel as Base


class Client(Base):
    """A Docker Client.

    Provides the methods to connect to a docker engine.

    Attributes:
        client (ClassVar[DockerClient]): A shared docker client session.
    """

    client: ClassVar[DockerClient] = None

    @classmethod
    def get_client(cls) -> DockerClient:
        """Return the docker client."""
        if cls.client is None:
            cls.client = docker.from_env()

        cls.client.ping()
        logger.debug("Docker connection successful.")

        return cls.client

    class Config:
        """Pydantic Configuration."""

        arbitrary_types_allowed = True
        json_encoders = {
            TemporaryFile: lambda temp: temp.name,
            Container: lambda container: container.name,
        }


class ContainerOptions(Base):
    """Docker Container Options.

    Provides common options needed to start a docker container.

    Attributes:
        container_timeout (ClassVar[int]): Timeout in seconds to wait for the container
            to stop before sending a SIGKILL.
        image (Optional[str]): Name of docker image.
        auto_remove (bool): Enable auto-removal of the container on daemon side when the
            container’s process exits.
        detach (bool): Run container in the background and return a Container object.
        mounts (List[DockerMount]): Specification for mounts to be added to the container.
        ports (Dict[int, int]): Ports to bind inside the container.
    """

    container_timeout: ClassVar[int] = 5

    image: Optional[str]
    auto_remove: bool = True
    detach: bool = True
    mounts: List[DockerMount] = list()
    ports: Dict[int, int] = dict()


class ContainerBase(Client):
    """Docker Container Base with default options and commonly used methods.

    Attributes:
        container_options (ContainerOptions): ContainerOptions Object.
        container (Optional[Container]): Holds an instance of a initiated docker container.
    """

    container_options: ContainerOptions = ContainerOptions()
    container: Optional[Container]

    @property
    def container_id(self) -> str:
        """Return container instance id."""
        return self.container.id

    @property
    def container_short_id(self) -> str:
        """Return container instance short id."""
        return self.container.short_id

    @property
    def container_name(self) -> str:
        """Return container instance name."""
        return self.container.name

    @property
    def container_timeout(self) -> int:
        """Return container timeout."""
        return self.container_options.container_timeout

    def show_follow_logs_command(self) -> None:
        """Print log message with docker container logs command."""
        logger.info(f"Run the following command to show ({self.container_name}) containers logs.")
        logger.info(f"docker container logs -f {self.container_name}")

    def expose_port(self, port: int) -> None:
        """Add ports to expose to the container options.

        Args:
            port (int): The port to expose.
        """
        self.container_options.ports[port] = port

    def start(self, show_log: bool = False) -> None:
        """Start a container instance.

        Args:
            show_log (bool): If True shows the containers logs.
        """
        client = self.get_client()

        self.container = client.containers.run(**self.container_options.dict())

        logger.debug(f"Running container {self.container_name} {self.container_short_id}.")

        if show_log:
            self.print_logs()

    def restart(self) -> None:
        """Restart the container instance."""
        logger.debug(f"Restarting container {self.container_name} {self.container_short_id}.")
        self.container.restart(timeout=self.container_timeout)

    def print_logs(self) -> None:
        """Print the container instance logs."""
        self.container.reload()

        for line in self.container.logs().decode().split("\n"):
            logger.debug(line)

    def stop(self, show_log: bool = False) -> None:
        """Stop a container instance.

        Args:
            show_log (bool): If True shows the containers logs.
        """
        logger.debug(f"Stopping container {self.container_name} {self.container_short_id}.")

        if show_log:
            self.print_logs()

        self.container.stop(timeout=self.container_timeout)

        logger.debug(f"Container {self.container_name} {self.container_short_id} Destroyed.")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['requests_whaor']

package_data = \
{'': ['*'], 'requests_whaor': ['templates/*']}

install_requires = \
['docker>=4.3.1,<5.0.0',
 'enlighten>=1.6.2,<2.0.0',
 'jinja2>=2.11.2,<3.0.0',
 'loguru>=0.5.2,<0.6.0',
 'more_itertools>=8.5.0,<9.0.0',
 'pydantic>=1.6.1,<2.0.0',
 'python-decouple>=3.3,<4.0',
 'requests[socks]>=2.24.0,<3.0.0']

setup_kwargs = {
    'name': 'requests-whaor',
    'version': '0.2.0',
    'description': 'Requests With High Availability Onion Router. For the filthiest web scrapers that have no time for rate-limits.',
    'long_description': "# **requests-whaor** [[ri-kwests](https://www.dictionary.com/browse/requests) [hawr](https://www.dictionary.com/browse/whore)]\n\n[**Requests**](https://requests.readthedocs.io) **+** [**Docker**](https://www.docker.com/) **+** [**HAproxy**](http://www.haproxy.org/) **+** [**Tor**](https://www.torproject.org/)\n\n**Requests** **W**ith **H**igh **A**vailability **O**nion **R**outer. For the filthiest web scrapers that have no time for rate-limits.\n\n[![Black](https://img.shields.io/badge/code%20style-black-black?style=for-the-badge&logo=appveyor)](https://github.com/psf/black)\n[![GitHub](https://img.shields.io/github/license/dgnsrekt/requests-whaor?style=for-the-badge)](https://raw.githubusercontent.com/dgnsrekt/requests-whaor/master/LICENSE)\n\n\n## Overview\n**requests-whaor** proxies GET requests through a local **Docker** network of **TOR** circuits. It takes care of starting and stopping a pool of **TOR** proxies behind an **HAproxy** load balancer, which acts as a round robin reverse proxy network. This will give each request a new IP address.  If you start having issues with the initial pool of IPs, **requests-whaor** can gather a new pool of IP addresses by restarting all **TOR** containers.\n\n## Install with pip\n```\npip install requests-whaor\n```\n\n## Install with [Poetry](https://python-poetry.org/)\n```\npoetry add requests-whaor\n```\n\n\n## [>> **Quickstart** / **Docs** <<](https://dgnsrekt.github.io/requests-whaor/quickstart)\n\n## Projects to highlight.\n* [**dperson's**](https://hub.docker.com/u/dperson) - [torproxy docker container](https://hub.docker.com/r/dperson/torproxy)\n* [**zet4's**](https://github.com/zet4) - [alpine-tor library](https://github.com/zet4/alpine-tor)\n* [torproject](https://www.torproject.org/)\n* [haproxy](https://hub.docker.com/_/haproxy)\n\n## Useful Docker commands.\n### If things get out of hand you may need these commands for debugging or killing containers.\n```\ndocker ps -q --filter ancestor=osminogin/tor-simple | xargs -L 1 docker logs --follow\n\ndocker ps -q --filter ancestor=osminogin/haproxy | xargs -L 1 docker logs --follow\n\ndocker stop $(docker ps -q --filter ancestor=osminogin/tor-simple)\n\ndocker stop $(docker ps -q --filter ancestor=haproxy)\n\ndocker network rm $(docker network ls -q -f name=whaornet)\n```\n\n## TODO\n* [ ] Testing.\n* [ ] More request methods if requested.\n* [ ] Options for using different Tor containers.\n* [ ] Options for different load balancer containers.\n\n## Contact Information\nTelegram = Twitter = Tradingview = Discord = @dgnsrekt\n\nEmail = dgnsrekt@pm.me\n",
    'author': 'dgnsrekt',
    'author_email': 'dgnsrekt@pm.me',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://dgnsrekt.github.io/requests-whaor/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



"""This module provides objects for managing the network balancer."""

from contextlib import contextmanager
from typing import Dict, List

from docker.models.containers import Container
from loguru import logger
from pydantic import BaseModel as Base

from .circuit import OnionCircuit
from .client import ContainerBase, ContainerOptions
from .mount import MountFile, MountPoint

HAPROXY_IMAGE = "haproxy:2.2.3"


class HAProxyOptions(Base):
    """Handles options for HAProxy docker instance.

    Attributes:
        max_connections (int): Maximum per-process number of concurrent connections.
        timeout_client (int): Maximum inactivity time on the client side.
        timeout_connect (int): Maximum time to wait for a connection attempt to a server
            to succeed.
        timeout_queue (int): Maximum time to wait in the queue for a connection slot
            to be free.
        timeout_server (int): Maximum inactivity time on the server side.
        listen_host_port (int): Frontend port to the proxy.
        backend_name (str): Name of Backend section.
        dashboard_bind_port (int): Port to open to reach the HAProxy dashboard.
        dashboard_refresh_rate (int): Refresh rate of the HAProxy dashboard page.
        onions (List[Container]): Each onion container that is connected to the whaornet.
    """

    max_connections: int = 4096

    timeout_client: int = 3600
    timeout_connect: int = 1
    timeout_queue: int = 5
    timeout_server: int = 3600

    listen_host_port: int = 8001

    backend_name: str = "onions"

    dashboard_bind_port: int = 9999
    dashboard_refresh_rate: int = 2

    onions: List[Container]

    class Config:
        """Pydantic Configuration."""

        arbitrary_types_allowed = True

    @property
    def ports(self) -> List[int]:
        """Ports which will be used to expose on the local network."""
        return [self.listen_host_port, self.dashboard_bind_port]


class Balancer(ContainerBase):
    """HAProxy Load Balancer.

    Attributes:
        haproxy_options (HAProxyOptions): HAProxy options object.
        container_options (ContainerOptions): Container options for the HA proxy instance.
    """

    haproxy_options: HAProxyOptions

    container_options: ContainerOptions = ContainerOptions(image=HAPROXY_IMAGE)

    class Config:
        """Pydantic Configuration."""

        arbitrary_types_allowed = True

    @property
    def address(self) -> str:
        """Return socks5 address to poxy requests through."""
        return f"socks5://localhost:{self.haproxy_options.listen_host_port}"

    @property
    def dashboard_address(self) -> str:
        """Return full dashboard address."""
        return f"http://localhost:{self.haproxy_options.dashboard_bind_port}"

    @property
    def proxies(self) -> Dict[str, str]:
        """Return proxies to mount onto a requests session."""
        return {
            "http": self.address,
            "https": self.address,
        }

    def add_mount_point(self, mount: MountFile) -> None:
        """Mount a volume into the HAProxy container.

        Args:
            mount (MountFile): File to mount between the container and local file system.
        """
        self.container_options.mounts.append(mount.mount)

    def display_settings(self) -> None:
        """Log config settings to stdout."""
        logger.debug(
            "\n==================="
            "\nOnion Load Balancer"
            "\n==================="
            "\n" + self.json(indent=4)
        )
        self.show_follow_logs_command()


@contextmanager
# pylint: disable=invalid-name
def OnionBalancer(onions: List[OnionCircuit], show_log: bool = False) -> Balancer:
    """Context manager which yields a started instance of an HAProxy docker container.

    Args:
        onions (List[OnionCircuit]): List of tor containers to load balance requests across.
        show_log (bool): If True shows the HAProxies logs on start and stop.

    Yields:
        Balancer: A started instance of a HAProxy docker container.
    """
    haproxy_options = HAProxyOptions(onions=onions)

    with MountPoint(
        template_name="haproxy.cfg",
        target_path="/usr/local/etc/haproxy/haproxy.cfg",
        template_variables=haproxy_options.dict(),
    ) as mount_point:

        try:
            balancer = Balancer(haproxy_options=haproxy_options)
            balancer.add_mount_point(mount_point)

            for port in haproxy_options.ports:
                balancer.expose_port(port)

            balancer.start(show_log=show_log)
            balancer.display_settings()

            yield balancer

        finally:
            balancer.stop(show_log=show_log)



"""This module provides objects for managing docker mount instances."""

from contextlib import contextmanager
from tempfile import _TemporaryFileWrapper as TemporaryFile
from tempfile import NamedTemporaryFile
from typing import Any, Dict, Optional

from docker.models.volumes import Volume as DockerVolume
from docker.types import Mount
from jinja2 import Environment, FileSystemLoader
from pydantic import validator

from .client import Client
from .paths import TEMPLATE_DIRECTORY, TEMPORARY_FILES_DIRECTORY


class MountFile(Client):
    """Represents a file to be binded to a docker container.

    Attributes:
        template_name (str): Name of the jinja template.
        target_path (str): Local file system path.
        volume_driver (str): Type of docker volume.
        temporary_file (Optional[TemporaryFile]): A mounted instance of a TemporaryFile.
        mount (Optional[Mount]): An instance of the DockerMount.
        template_variables (Optional[Dict[Any, Any]]): Jinja template variables.
    """

    template_name: str
    target_path: str
    volume_driver: str = "local"

    temporary_file: Optional[TemporaryFile]
    mount: Optional[Mount]
    template_variables: Optional[Dict[Any, Any]]

    class Config:
        """Pydantic Configuration."""

        arbitrary_types_allowed = True
        json_encoders = {TemporaryFile: lambda temp: temp.name, DockerVolume: lambda vol: vol.name}

    @validator("template_name")
    def _template_must_exist(  # pylint: disable=no-self-argument,no-self-use
        cls, template_name: str
    ) -> str:
        """Check if template exists in template directory."""
        template_path = TEMPLATE_DIRECTORY / template_name
        if template_path.exists():
            return template_name

        raise ValueError(f"{template_name} does not exist.")

    @property
    def volume_name(self) -> str:
        """Name of the volume."""
        return self.template_name.replace(".", "_") + "_volume"

    @property
    def source_path(self) -> str:
        """Temporary file path name."""
        return self.temporary_file.name

    def _render_template(self) -> str:
        """Render jinja template."""
        env = Environment(loader=FileSystemLoader(TEMPLATE_DIRECTORY), keep_trailing_newline=True)
        template = env.get_template(self.template_name)

        if self.template_variables:
            return template.render(self.template_variables)

        return template.render()

    def _create_source_file(self) -> None:
        """Create temp file for storing render template output."""
        self.temporary_file = NamedTemporaryFile(dir=TEMPORARY_FILES_DIRECTORY, suffix=".conf")

    def _generate_source_file(self) -> None:
        """Generate source file."""
        render_data = self._render_template()
        self._create_source_file()
        self.temporary_file.write(render_data.encode())
        self.temporary_file.seek(0)

    def start(self) -> None:
        """Start the volume mount."""
        self._generate_source_file()
        self.mount = Mount(
            target=self.target_path, source=self.source_path, read_only=True, type="bind"
        )

    def stop(self) -> None:
        """Stop the volume mount."""
        if self.temporary_file:
            self.temporary_file.close()


@contextmanager
# pylint: disable=invalid-name
def MountPoint(
    *, template_name: str, target_path: str, template_variables: Optional[Dict[Any, Any]] = None
) -> MountFile:
    """Context manager which yields a prepared instance of a docker volume.

    Args:
        template_name (str): Name of the jinja template.
        target_path (str): Local file system path.
        template_variables (Optional[Dict[Any, Any]]): Jinja template variables.
    """
    mount_file = MountFile(
        template_name=template_name, target_path=target_path, template_variables=template_variables
    )

    try:
        mount_file.start()
        yield mount_file

    finally:
        mount_file.stop()
